@@ -18,7 +18,7 @@
 
 static int    jumping;
 static int    extcommand; // [lc]
-static char    jcmd[100];
+static char   jcmd[100];
 static int    jidx;
 static int    cursor_timeout;
 static gboolean displaying_textwindow = FALSE;
@@ -61,6 +61,48 @@ static void qiv_cancel_cursor_timeout(qiv_image *q)
   }
 }
 
+static void qiv_grab(qiv_image *q)
+{
+#if GDK_MAJOR_VERSION < 3
+   gdk_keyboard_grab(q->win, FALSE, GDK_CURRENT_TIME);
+   gdk_pointer_grab(q->win, FALSE,
+       GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_ENTER_NOTIFY_MASK |
+       GDK_LEAVE_NOTIFY_MASK | GDK_POINTER_MOTION_MASK, NULL, NULL, GDK_CURRENT_TIME);
+#else
+   int ret;
+   GdkSeat *seat;
+   seat = gdk_display_get_default_seat(gdk_display_get_default());
+   ret = gdk_seat_grab(seat, q->win, GDK_SEAT_CAPABILITY_KEYBOARD,
+                         TRUE, NULL, NULL, NULL, NULL);
+   if(ret != GDK_GRAB_SUCCESS)
+      printf("grab failed%d\n",ret);
+#endif
+
+}
+
+void qiv_ungrab()
+{
+#if GDK_MAJOR_VERSION < 3
+   gdk_pointer_ungrab(GDK_CURRENT_TIME);
+   gdk_keyboard_ungrab(GDK_CURRENT_TIME);
+#else
+   GdkSeat *seat;
+   seat = gdk_display_get_default_seat(gdk_display_get_default());
+   gdk_seat_ungrab(seat);
+#endif
+}
+
+static void qiv_window_get_pointer(qiv_image *q, gint *xcur, gint *ycur)
+{
+#if GDK_MAJOR_VERSION < 3
+   gdk_window_get_pointer(q->win, xcur, ycur, NULL);
+#else
+   GdkSeat *seat;
+   seat = gdk_display_get_default_seat(gdk_display_get_default());
+   gdk_window_get_device_position(q->win, gdk_seat_get_pointer(seat), xcur, ycur, NULL);
+#endif
+}
+
 static void qiv_drag_image(qiv_image *q, int move_to_x, int move_to_y,
                            const char *action_msg, const char *stalled_msg)
 {
@@ -204,6 +246,13 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
         else
           update_image(q, FULL_REDRAW);
       }
+      // redraw magnify windows on exposed event if it exists
+      if (magnify && !fullscreen && !magnify_img.exposed) {
+          gint xcur, ycur;
+          qiv_window_get_pointer(q, &xcur, &ycur);
+          update_magnify(q, &magnify_img, REDRAW, xcur, ycur);
+          magnify_img.exposed = 1;
+      }
       q->exposed = 1;
       break;
 
@@ -211,17 +260,13 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
       if (magnify && !fullscreen) {
         gdk_window_hide(magnify_img.win);
       }
-      gdk_pointer_ungrab(CurrentTime);
-      gdk_keyboard_ungrab(CurrentTime);
+      qiv_ungrab();
       break;
 
     case GDK_ENTER_NOTIFY:
       if(do_grab || (fullscreen && !disable_grab))
       {
-        gdk_keyboard_grab(q->win, FALSE, CurrentTime);
-        gdk_pointer_grab(q->win, FALSE,
-            GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_ENTER_NOTIFY_MASK |
-            GDK_LEAVE_NOTIFY_MASK | GDK_POINTER_MOTION_MASK, NULL, NULL, CurrentTime);
+        qiv_grab(q);
       }
 
     case GDK_CONFIGURE:
@@ -284,7 +329,7 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
         if (magnify && !fullscreen) {
           gint xcur, ycur;
           if (ev->motion.is_hint) {
-            gdk_window_get_pointer(q->win, &xcur, &ycur, NULL);
+            qiv_window_get_pointer(q, &xcur, &ycur);
             update_magnify(q, &magnify_img,REDRAW, xcur, ycur); // [lc]
           }
           // update_magnify(q, &magnify_img,REDRAW, ev->motion.x,  ev->motion.y);
@@ -799,7 +844,7 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
             if (magnify && !fullscreen) {
               gint xcur, ycur;
               magnify_img.zoom *= 1.1;
-              gdk_window_get_pointer(q->win, &xcur, &ycur, NULL);
+              qiv_window_get_pointer(q, &xcur, &ycur);
               update_magnify(q, &magnify_img,REDRAW, xcur, ycur); // [lc]
             }
             else
@@ -821,7 +866,7 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
                 gint xcur, ycur;
                 magnify_img.zoom /= 1.1;
                 if (magnify_img.zoom < 2) magnify_img.zoom = 2.0;
-                gdk_window_get_pointer(q->win, &xcur, &ycur, NULL);
+                qiv_window_get_pointer(q, &xcur, &ycur);
                 update_magnify(q, &magnify_img,REDRAW, xcur, ycur);
               }
             }
@@ -895,7 +940,10 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
 
           case 'B':
             snprintf(infotext, sizeof infotext, "(More brightness)");
-            q->mod.brightness += 8;
+            if(++q->mod.brightness > 50)
+            {
+                q->mod.brightness= 50;
+            }
             update_image(q, REDRAW);
             break;
 
@@ -903,7 +951,10 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
 
           case 'b':
             snprintf(infotext, sizeof infotext, "(Less brightness)");
-            q->mod.brightness -= 8;
+            if(--q->mod.brightness < -50)
+            {
+                q->mod.brightness= -50;
+            }
             update_image(q, REDRAW);
             break;
 
@@ -911,7 +962,10 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
 
           case 'C':
             snprintf(infotext, sizeof infotext, "(More contrast)");
-            q->mod.contrast += 8;
+            if(++q->mod.contrast > 100)
+            {
+               q->mod.contrast = 100;
+            }
             update_image(q, REDRAW);
             break;
 
@@ -919,7 +973,10 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
 
           case 'c':
             snprintf(infotext, sizeof infotext, "(Less contrast)");
-            q->mod.contrast -= 8;
+            if(--q->mod.contrast < -50)
+            {
+                q->mod.contrast = -50;
+            }
             update_image(q, REDRAW);
             break;
 
@@ -927,7 +984,10 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
 
           case 'G':
             snprintf(infotext, sizeof infotext, "(More gamma)");
-            q->mod.gamma += 8;
+            if(++q->mod.gamma > 100)
+            {
+                q->mod.gamma = 100;
+            }
             update_image(q, REDRAW);
             break;
 
@@ -935,7 +995,10 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
 
           case 'g':
             snprintf(infotext, sizeof infotext, "(Less gamma)");
-            q->mod.gamma -= 8;
+            if(--q->mod.gamma < -50)
+            {
+                q->mod.gamma = -50;
+            }
             update_image(q, REDRAW);
             break;
 
@@ -1116,7 +1179,7 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
                 gdk_window_hide(magnify_img.win);
               } else {
                 setup_magnify(q, &magnify_img);
-                gdk_window_get_pointer(q->win, &xcur, &ycur, NULL);
+                qiv_window_get_pointer(q, &xcur, &ycur);
                 update_magnify(q, &magnify_img, REDRAW, xcur, ycur); // [lc]
                 qiv_disable_mouse_events(q);
               }
@@ -1150,26 +1213,11 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
             }
             if((!fullscreen && do_grab) || (fullscreen && !disable_grab))
             {
-#if GDK_MAJOR_VERSION < 3
-              gdk_pointer_grab(q->win, FALSE,
-                  GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_ENTER_NOTIFY_MASK |
-                  GDK_LEAVE_NOTIFY_MASK | GDK_POINTER_MOTION_MASK, NULL, NULL, CurrentTime);
-              gdk_keyboard_grab(q->win, FALSE, CurrentTime);
-#else
-              GdkDisplay* display = gdk_display_get_default();
-              GdkDeviceManager* device_manager = gdk_display_get_device_manager(display);
-              GdkDevice* pointer = gdk_device_manager_get_client_pointer(device_manager);
-              gdk_device_grab(pointer, q->win, GDK_OWNERSHIP_NONE, FALSE,
-                  GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_ENTER_NOTIFY_MASK |
-                  GDK_LEAVE_NOTIFY_MASK | GDK_POINTER_MOTION_MASK, NULL, CurrentTime);
-              GdkDevice* keyboard = gdk_device_get_associated_device(pointer);
-              gdk_device_grab(keyboard, q->win, GDK_OWNERSHIP_NONE, FALSE, FALSE, NULL, CurrentTime);
-#endif
+              qiv_grab(q);
             }
             else
             {
-              gdk_pointer_ungrab(CurrentTime);
-              gdk_keyboard_ungrab(CurrentTime);
+              qiv_ungrab();
             }
             update_image(q, MIN_REDRAW);
             break;
